""" Wrapper script for awscli which connects to containers running in
    ecs seamlessly """
import click
import logging
import os
import re
import inquirer
import signal
from ecs_connect.version import __version__
from ecs_connect.config import ECSConfig
from ecs_connect.ecs import ECSHandler
from ecs_connect.ssm import SSMHandler
from configparser import RawConfigParser

@click.command()
@click.option('--profile', help="Name of the profile to use in ~/.ecs-connect. \
If none is provided, then the default profile will be used.\n")
@click.option('--awsprofile', help="Name of the aws profile to use form ecs-connect. \
If none is provided, then the default aws profile will be used.\n")
@click.option('--cluster', help="Name of the ECS cluster. \
If provided, then parameter from profile will be overridden.\n")
@click.option('--service', help="Name of the service. \
If provided, then parameter from profile will be overridden.\n")
@click.option('--task', help="Name of the task. \
If provided, then parameter from profile will be overridden.\n")
@click.option('--exec_cmd', help="Initilization command to run with ecs exec. \
If provided, then parameter from profile will be overridden.\n")
@click.option('--container', help="Name of the container. \
If provided, then parameter from profile will be overridden.\n")
@click.option('--bastion', help="Instance Id of the bastion node, \
required for task running in FARGATE. \
If provided, then parameter from profile will be overridden.\n")
@click.option('--cmd', help="Initilization command to run. \
If provided, then parameter from profile will be overridden.\n")
@click.option('-a', '--all', is_flag=True,
              help='Displays all running containers\n')
@click.option('-V', '--version', is_flag=True,
              help='Displays version number\n')
@click.option('-v', '--verbose', is_flag=True, help='Enables verbose mode')
@click.option('-d', '--debug', is_flag=True, help='Enables debug mode')
@click.option('-i', '--interactive', is_flag=True, help='Interactive mode')
def main(profile, awsprofile, cluster, service, task, container, bastion, exec_cmd, cmd, all, version, verbose, debug, interactive):
    if version:
        print(__version__)
        exit(0)

    # Set up logging
    logger = logging.getLogger('ecs-connect')
    logger.setLevel(logging.DEBUG)
    handler = logging.StreamHandler()
    handler.setLevel(logging.WARN)
    formatter = logging.Formatter('%(levelname)s - %(message)s')
    handler.setFormatter(formatter)
    if verbose:
        handler.setLevel(logging.INFO)
    if debug:
        handler.setLevel(logging.DEBUG)
    logger.addHandler(handler)

    if not interactive:
        ecs_config = ECSConfig(logger)
        if not profile:
            profile = "default"
        if not cluster:
            cluster = ecs_config.get_cluster(profile)
        if not awsprofile:
            awsprofile = ecs_config.get_awsprofile(profile)
        if not task:
            task = ecs_config.get_task(profile)
        if not exec_cmd:
            exec_cmd = ecs_config.get_exec_cmd(profile)
        if not container:
            container = ecs_config.get_container(profile, exec_cmd)
        if task is None:
            if not service:
                service = ecs_config.get_service(profile)
        else:
            service = None
        if not bastion:
            bastion = ecs_config.get_bastion(profile)
        if not cmd:
            cmd = ecs_config.get_cmd(profile, exec_cmd)
        ssh_user = ecs_config.get_ssh_user(profile)
        ssh_key = ecs_config.get_ssh_key(profile)
        ssh_port = ecs_config.get_ssh_port(profile)

        ecs = ECSHandler(awsprofile, cluster, service, task, container, bastion, logger, cmd, exec_cmd)
        if exec_cmd is not None:
            ecs.exec()
        else:
            instance_id, bastion_enabled = ecs.get_ec2_instance_id()
            ssm = SSMHandler(awsprofile, instance_id, service, bastion_enabled, bastion, ssh_user, ssh_key, ssh_port, logger)
            ssm.run(cmd, all)
    else:
        config_path = os.path.expanduser('~') + '/.aws/config'
        _value = RawConfigParser()
        _value.read(config_path)
        aws_profiles = _value.sections()
        profiles = map( str, aws_profiles)
        profiles = list(map( lambda x: re.sub("profile ", "", x), profiles))
        questions = [
            inquirer.List('aws',
                        message="Select AWS Profile",
                        choices=profiles,
                    ),
        ]
        answers = inquirer.prompt(questions)
        ecs = ECSHandler(answers["aws"], '', '', '', '', '', logger, '', 'bash')
        ecs.interactive()

if __name__ == "__main__":
    main()



"""Main module."""

import boto3
import subprocess
import time


class SSMHandler():
    """ SSM handler  """
    def __init__(self, awsprofile, ec2_id, service, bastion_enabled, bastion_id, ssh_user, ssh_key, ssh_port, logger):
        self.awsprofile = awsprofile
        self.ec2_id = ec2_id
        self.service = service
        self.bastion_enabled = bastion_enabled
        self.bastion_id = bastion_id
        self.ssh_user = ssh_user
        self.ssh_key = ssh_key
        self.ssh_port = ssh_port
        self.logger = logger

        self.ssm_client = boto3.session.Session(profile_name=self.awsprofile).client('ssm')

    def get_conatiners(self, all):
        if all:
            exec_cmd = f'sudo docker ps'
        else:
            exec_cmd = f'sudo docker ps -f name={self.service}'
        response = self.ssm_client.send_command(
                    InstanceIds=[self.ec2_id],
                    DocumentName="AWS-RunShellScript",
                    Parameters={'commands': [exec_cmd]}, )
        command_id = response['Command']['CommandId']

        while True:
            time.sleep(1)
            output = self.ssm_client.get_command_invocation(
                  CommandId=command_id,
                  InstanceId=self.ec2_id,
                )
            if output['Status'] != "InProgress":
                break

        running_container = output['StandardOutputContent'].count('\n') - 1
        if running_container <= 0:
            self.logger.error("No container running with name %s.\
                \nUse --all/-a to display all running container.",
                              self.service)
            exit(1)

        if running_container == 1:
            lines = output['StandardOutputContent'].split('\n')
            conatiner_details = lines[1].split()
            return conatiner_details[0]

        print(output['StandardOutputContent'])
        container_id = input('Please enter conatiner id: ')
        return container_id

    def start_session(self, container_id, exec_cmd):
        command = f'aws --profile {self.awsprofile} ssm start-session --target {self.ec2_id} \
        --document-name AWS-StartInteractiveCommand \
        --parameters command="sudo docker exec -it {container_id} {exec_cmd}"'
        self.logger.info("Running: %s", command)

        subprocess.call(command, shell=True)

    def start_bastion_session(self, exec_cmd):
        command = f'aws --profile {self.awsprofile} ssm start-session --target {self.bastion_id} \
        --document-name AWS-StartInteractiveCommand \
        --parameters command="ssh -i {self.ssh_key} -p {self.ssh_port} {self.ssh_user}@{self.ec2_id}"'
        self.logger.info("Running: %s", command)

        subprocess.call(command, shell=True)

    def run_directly(self, exec_cmd, all):
        container_id = self.get_conatiners(all)
        self.start_session(container_id, exec_cmd)

    def run_through_bastion(self, exec_cmd, all):
        self.start_bastion_session(exec_cmd)

    def run(self, exec_cmd, all):
        if self.bastion_enabled:
            self.run_through_bastion(exec_cmd, all)
        else:
            self.run_directly(exec_cmd, all)



#!/usr/bin/env python

"""The setup script."""

import os
import codecs
import re
from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

def read(*parts):
    with codecs.open(os.path.join(HERE, *parts), 'r') as fp:
        return fp.read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open(os.path.join(HERE, "requirements.txt"), "r") as fh:
    INSTALL_REQUIRES = [line.strip() for line in fh]

with open(os.path.join(HERE, "requirements_dev.txt"), "r") as fh:
    SETUP_REQUIRES = [line.strip() for line in fh]


setup(
    author="Saurabh Jambhule",
    author_email='saurabhjambhule@gmail.com',
    python_requires='>=3.5',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Seamlessly connect to containers running in ECS.",
    entry_points={
        'console_scripts': [
            'ecs_connect=ecs_connect.cli:main',
        ],
    },
    install_requires=INSTALL_REQUIRES,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    keywords=[
        "ecs_connect",
        "ecs",
    ],
    name='ecs_connect',
    packages=find_packages(include=['ecs_connect', 'ecs_connect.*']),
    setup_requires=SETUP_REQUIRES,
    url='https://github.com/saurabhjambhule/ecs_connect',
    version=find_version("ecs_connect/version.py"),
    test_suite='tests',
    zip_safe=False,
)
